/*
==========================================================================

  html.y: PCYACC grammar description for HTML

  (c) COPYRIGHT, PCYACC ABRAXAS SOFTWARE, INC.
  version 1.0
  by Shuming Tan

  Reference: HTML Manual of Style by Larry Aronson,
             published by ZD press
===========================================================================
*/
/* file public.h contains the macros and data types refered by union */
%{
#include "public.h"
%}

%union
{
  struct tag_struct tag;
  unsigned char text[TEXT_LEN];
}

%token A_BEGIN_TAG A_END_TAG
       ADDRESS_BEGIN_TAG ADDRESS_END_TAG
       B_BEGIN_TAG B_END_TAG
       BASE_TAG
       BLOCKQUOTE_BEGIN_TAG BLOCKQUOTE_END_TAG
       BODY_BEGIN_TAG BODY_END_TAG
       BR_TAG
       CITE_BEGIN_TAG CITE_END_TAG
       CODE_BEGIN_TAG CODE_END_TAG
       DD_TAG
       DIR_BEGIN_TAG DIR_END_TAG
       DL_BEGIN_TAG DL_END_TAG
       DT_TAG
       EM_BEGIN_TAG EM_END_TAG
       FORM_BEGIN_TAG FORM_END_TAG
       H1_BEGIN_TAG H1_END_TAG
       H2_BEGIN_TAG H2_END_TAG
       H3_BEGIN_TAG H3_END_TAG
       H4_BEGIN_TAG H4_END_TAG
       H5_BEGIN_TAG H5_END_TAG
       H6_BEGIN_TAG H6_END_TAG
       HEAD_BEGIN_TAG HEAD_END_TAG
       HR_TAG
       HTML_BEGIN_TAG HTML_END_TAG
       I_BEGIN_TAG I_END_TAG
       IMG_TAG
       INPUT_TAG
       ISINDEX_TAG
       KBD_BEGIN_TAG KBD_END_TAG
       LI_TAG
       LINK_TAG
       MENU_BEGIN_TAG MENU_END_TAG
       OL_BEGIN_TAG OL_END_TAG
       OPTION_TAG
       P_TAG
       PRE_BEGIN_TAG PRE_END_TAG
       SAMP_BEGIN_TAG SAMP_END_TAG
       SELECT_BEGIN_TAG SELECT_END_TAG
       STRONG_BEGIN_TAG STRONG_END_TAG
       TEXTAREA_BEGIN_TAG TEXTAREA_END_TAG
       TITLE_BEGIN_TAG TITLE_END_TAG
       TT_BEGIN_TAG TT_END_TAG
       UL_BEGIN_TAG UL_END_TAG
       VAR_BEGIN_TAG VAR_END_TAG
%token PLAIN_TEXT
%left  A_BEGIN_TAG A_END_TAG
       ADDRESS_BEGIN_TAG ADDRESS_END_TAG
       B_BEGIN_TAG B_END_TAG
       BASE_TAG
       BLOCKQUOTE_BEGIN_TAG BLOCKQUOTE_END_TAG
       BODY_BEGIN_TAG BODY_END_TAG
       BR_TAG
       CITE_BEGIN_TAG CITE_END_TAG
       CODE_BEGIN_TAG CODE_END_TAG
       DD_TAG
       DIR_BEGIN_TAG DIR_END_TAG
       DL_BEGIN_TAG DL_END_TAG
       DT_TAG
       EM_BEGIN_TAG EM_END_TAG
       FORM_BEGIN_TAG FORM_END_TAG
       H1_BEGIN_TAG H1_END_TAG
       H2_BEGIN_TAG H2_END_TAG
       H3_BEGIN_TAG H3_END_TAG
       H4_BEGIN_TAG H4_END_TAG
       H5_BEGIN_TAG H5_END_TAG
       H6_BEGIN_TAG H6_END_TAG
       HEAD_BEGIN_TAG HEAD_END_TAG
       HR_TAG
       HTML_BEGIN_TAG HTML_END_TAG
       I_BEGIN_TAG I_END_TAG
       IMG_TAG
       INPUT_TAG
       ISINDEX_TAG
       KBD_BEGIN_TAG KBD_END_TAG
       LI_TAG
       LINK_TAG
       MENU_BEGIN_TAG MENU_END_TAG
       OL_BEGIN_TAG OL_END_TAG
       OPTION_TAG
       P_TAG
       PRE_BEGIN_TAG PRE_END_TAG
       SAMP_BEGIN_TAG SAMP_END_TAG
       SELECT_BEGIN_TAG SELECT_END_TAG
       STRONG_BEGIN_TAG STRONG_END_TAG
       TEXTAREA_BEGIN_TAG TEXTAREA_END_TAG
       TITLE_BEGIN_TAG TITLE_END_TAG
       TT_BEGIN_TAG TT_END_TAG
       UL_BEGIN_TAG UL_END_TAG
       VAR_BEGIN_TAG VAR_END_TAG
       PLAIN_TEXT
%left error

%start html     
%%
address
  : ADDRESS_BEGIN_TAG address_items ADDRESS_END_TAG
  | ADDRESS_BEGIN_TAG address_items error
  ;

address_items
  :               address_item
  | address_items address_item
  ;

address_item
  : a
  ;

a
  : A_BEGIN_TAG a_items A_END_TAG
  | A_BEGIN_TAG a_items error
  ;

a_items
  :         a_item
  | a_items a_item
  ;

a_item
  : PLAIN_TEXT
  | b
  | br
  | cite
  | code
  | em
  | i
  | img
  | kbd
  | samp
  | strong
  | tt
  | var
  ;

b
  : B_BEGIN_TAG b_items B_END_TAG
  | B_BEGIN_TAG b_items error
  ;

b_items 
  :         b_item
  | b_items b_item
  ;

b_item
  : PLAIN_TEXT
  | a
  | br
  | cite
  | code
  | em
  | i
  | img
  | kbd
  | samp
  | strong
  | tt
  | var
  ;

Base
  :
  | BASE_TAG
  ;

                                                                                                                                                                                                       
blockquote
  : BLOCKQUOTE_BEGIN_TAG blockquote_items BLOCKQUOTE_END_TAG
  | BLOCKQUOTE_BEGIN_TAG blockquote_items error
  ;

blockquote_items
  :                  blockquote_item
  | blockquote_items blockquote_item
  ;

blockquote_item
  : PLAIN_TEXT
  | address
  | blockquote
  | dir
  | dl
  | form
  | h1
  | h2
  | h3
  | h4
  | h5
  | h6
  | hr
  | menu
  | ol
  | p
  | pre
  | ul
  ;

dir
  : DIR_BEGIN_TAG dir_items DIR_END_TAG
  | DIR_BEGIN_TAG dir_items error
  ;

dir_items
  :           dir_item
  | dir_items dir_item
  ;

dir_item
  : li
  ;

body
  : BODY_BEGIN_TAG body_items BODY_END_TAG
  | BODY_BEGIN_TAG body_items error
  ;

body_items
  :
  | body_items body_item
  ;

body_item
  : PLAIN_TEXT
  | a
  | address 
  | b
  | blockquote
  | br
  | cite
  | code
  | dir
  | dl
  | form
  | em
  | h1
  | h2
  | h3
  | h4
  | h5
  | h6
  | hr
  | i
  | img
  | kbd
  | menu
  | ol
  | p
  | pre
  | samp
  | strong
  | tt
  | ul
  | var
  ;

br
  : BR_TAG
  ;

cite
  : CITE_BEGIN_TAG cite_items CITE_END_TAG
  | CITE_BEGIN_TAG cite_items error
  ;

cite_items
  : cite_item
  | cite_items cite_item
  ;

cite_item
  : PLAIN_TEXT
  | a
  | b
  | br
  | code
  | em
  | i
  | img
  | kbd 
  | samp
  | strong
  | tt
  | var
  ;

code
  : CODE_BEGIN_TAG code_items CODE_END_TAG
  | CODE_BEGIN_TAG code_items error
  ;

code_items
  : code_item
  | code_items code_item
  ;

code_item
  : a
  | b
  | br
  | cite
  | em
  | i
  | img
  | kbd 
  | samp
  | strong
  | tt
  | var
  ;

dd
  : DD_TAG dd_items
  ;

dd_items
  : dd_item
  | dd_items dd_item
  ;

dd_item
  : PLAIN_TEXT
  | a
  | b
  | blockquote
  | br
  | cite
  | code
  | dd
  | dl
  | em
  | form
  | i
  | img
  | menu
  | ol
  | p
  | pre
  | samp
  | strong
  | tt
  | ul
  | var
  ;
 
dl
  : DL_BEGIN_TAG dl_items DL_END_TAG
  | DL_BEGIN_TAG dl_items error
  ;

dl_items
  : dl_item
  | dl_items dl_item
  ;

dl_item
  : dd
  | dt
  ;

dt
  : DT_TAG dt_items
  ;

dt_items
  : dt_item
  | dt_items dt_item
  ;

dt_item
  : a
  | b
  | br
  | cite
  | code
  | em
  | i
  | img
  | kbd
  | samp
  | strong
  | tt
  | var
  ;

em
  : EM_BEGIN_TAG em_items EM_END_TAG
  | EM_BEGIN_TAG em_items error
  ;

em_items
  : em_item
  | em_items em_item
  ;

em_item
  : a
  | b
  | br
  | cite
  | code
  | i
  | img
  | kbd
  | samp
  | strong
  | tt
  | var
  ;
 
form
  : FORM_BEGIN_TAG form_items FORM_END_TAG
  | FORM_BEGIN_TAG form_items error
  ;

form_items
  : form_item
  | form_items form_item
  ;

form_item
  : address
  | blockquote
  | dir
  | dl
  | h1
  | h2
  | h3
  | h4
  | h5
  | h6
  | hr
  | input
  | menu
  | ol
  | p
  | pre
  | select
  | textarea
  | ul
  ;

h1
  : H1_BEGIN_TAG h1_items H1_END_TAG
  | H1_BEGIN_TAG h1_items error
  ;

h1_items
  : h1_item
  | h1_items h1_item
  ;

h1_item
  : PLAIN_TEXT
  | a
  | b
  | br
  | cite
  | code
  | em
  | i
  | img
  | kbd 
  | samp
  | strong
  | tt
  | var
  ;

h2
  : H2_BEGIN_TAG h2_items H2_END_TAG
  | H2_BEGIN_TAG h2_items error
  ;

h2_items
  : h2_item
  | h2_items h2_item
  ;

h2_item
  : PLAIN_TEXT
  | a
  | b
  | br
  | cite
  | code
  | em
  | i
  | img
  | kbd 
  | samp
  | strong
  | tt
  | var
  ;

h3
  : H3_BEGIN_TAG h3_items H3_END_TAG
  | H3_BEGIN_TAG h3_items error
  ;

h3_items
  : h3_item
  | h3_items h3_item
  ;

h3_item
  : PLAIN_TEXT
  | a
  | b
  | br
  | cite
  | code
  | em
  | i
  | img
  | kbd 
  | samp
  | strong
  | tt
  | var
  ;

h4
  : H4_BEGIN_TAG h4_items H4_END_TAG
  | H4_BEGIN_TAG h4_items error
  ;

h4_items
  : h4_item
  | h4_items h4_item
  ;

h4_item
  : PLAIN_TEXT
  | a
  | b
  | br
  | cite
  | code
  | em
  | i
  | img
  | kbd 
  | samp
  | strong
  | tt
  | var
  ;

h5
  : H5_BEGIN_TAG h5_items H5_END_TAG
  | H5_BEGIN_TAG h5_items error
  ;

h5_items
  : h5_item
  | h5_items h5_item
  ;

h5_item
  : PLAIN_TEXT
  | a
  | b
  | br
  | cite
  | code
  | em
  | i
  | img
  | kbd 
  | samp
  | strong
  | tt
  | var
  ;

h6
  : H6_BEGIN_TAG h6_items H6_END_TAG
  | H6_BEGIN_TAG h6_items error
  ;

h6_items
  : h6_item
  | h6_items h6_item
  ;

h6_item
  : PLAIN_TEXT
  | a
  | b
  | br
  | cite
  | code
  | em
  | i
  | img
  | kbd 
  | samp
  | strong
  | tt
  | var
  ;

head
  : HEAD_BEGIN_TAG Title Base Isindex Link HEAD_END_TAG
  | HEAD_BEGIN_TAG Title Base Isindex Link error
  ;

hr
  : HR_TAG
  ;

html
  : HTML_BEGIN_TAG head body HTML_END_TAG
  | HTML_BEGIN_TAG head body error
  |                head body
  | error
  ;

i
  : I_BEGIN_TAG i_items I_END_TAG
  | I_BEGIN_TAG i_items error
  ;

i_items
  :         i_item
  | i_items i_item
  ;

i_item
  : PLAIN_TEXT
  | a
  | b
  | br
  | cite
  | code
  | em
  | img
  | kbd
  | samp
  | strong
  | tt
  | var
  ;

img
  : IMG_TAG
  ;

input
  : INPUT_TAG
  ;

Isindex
  :
  | ISINDEX_TAG
  ;

kbd
  : KBD_BEGIN_TAG kbd_items KBD_END_TAG
  | KBD_BEGIN_TAG kbd_items error
  ;

kbd_items
  : kbd_item
  | kbd_items kbd_item
  ;

kbd_item
  : PLAIN_TEXT
  | a
  | b
  | br
  | cite
  | code
  | em
  | i
  | img
  | samp
  | strong
  | tt
  | var
  ;

li
  : LI_TAG li_items
  ;

li_items
  :          li_item
  | li_items li_item
  ;

li_item
  : PLAIN_TEXT
  | a
  | b
  | blockquote
  | br
  | cite
  | code
  | dir
  | dl
  | em
  | form
  | i
  | img
  | kbd
  | menu
  | ol
  | p
  | pre
  | strong
  | tt
  | ul
  | var
  ;

Link
  :
  | LINK_TAG
  ;


menu
  : MENU_BEGIN_TAG menu_items MENU_END_TAG
  | MENU_BEGIN_TAG menu_items error
  ;

menu_items
  : menu_item
  | menu_items menu_item
  ;

menu_item
  : li
  ;

ol
  : OL_BEGIN_TAG ol_items OL_END_TAG
  ;

ol_items
  : ol_item
  | ol_items ol_item
  ;

ol_item
  : li
  ;

option
  : OPTION_TAG
  ;

p
  : P_TAG
  ;

pre
  : PRE_BEGIN_TAG pre_items PRE_END_TAG
  | PRE_BEGIN_TAG pre_items error
  ;

pre_items
  : pre_item
  | pre_items pre_item
  ;

pre_item
  : PLAIN_TEXT
  | a
  | b
  | br
  | cite
  | code
  | em
  | hr
  | i
  | img
  | kbd
  | samp
  | strong
  | tt
  | var
  ;
                                                                                          
samp
  : SAMP_BEGIN_TAG samp_items SAMP_END_TAG
  | SAMP_BEGIN_TAG samp_items error
  ;

samp_items
  : samp_item
  | samp_items samp_item
  ;

samp_item
  : a
  | b
  | br
  | cite
  | code
  | em
  | i
  | img
  | kbd
  | strong
  | tt
  | var
  ;

select
  : SELECT_BEGIN_TAG select_items SELECT_END_TAG
  | SELECT_BEGIN_TAG select_items error
  | error            select_items SELECT_END_TAG
  ;

select_items
  : select_item
  | select_items select_item
  ;

select_item
  : option 
  ;

strong
  : STRONG_BEGIN_TAG strong_items STRONG_END_TAG
  | STRONG_BEGIN_TAG strong_items error
  ;

strong_items
  : strong_item
  | strong_items strong_item
  ;

strong_item
  : PLAIN_TEXT
  | a
  | b
  | br
  | cite
  | code
  | em
  | i
  | img
  | kbd
  | samp
  | tt
  | var
  ;

textarea
  : TEXTAREA_BEGIN_TAG PLAIN_TEXT TEXTAREA_END_TAG
  | error              PLAIN_TEXT TEXTAREA_END_TAG
  | TEXTAREA_BEGIN_TAG PLAIN_TEXT error
  ;


Title
  : TITLE_BEGIN_TAG PLAIN_TEXT TITLE_END_TAG
  ;

tt
  : TT_BEGIN_TAG tt_items TT_END_TAG
  | TT_BEGIN_TAG tt_items error
  ;

tt_items
  : a
  | b
  | br
  | cite
  | code
  | em
  | i
  | img
  | kbd
  | samp
  | strong
  | var
  ;

ul
  : UL_BEGIN_TAG ul_items UL_END_TAG
  | error        ul_items UL_END_TAG
  | UL_BEGIN_TAG ul_items error
  ;

ul_items
  : ul_item
  | ul_items ul_item
  ;

ul_item
  : li
  ;
var
  : VAR_BEGIN_TAG var_items VAR_END_TAG
  | error         var_items VAR_END_TAG
  ; 

var_items
  : var_item
  | var_items var_item
  ;

var_item
  : PLAIN_TEXT
  | a
  | b
  | br
  | cite
  | code
  | em
  | i
  | img
  | kbd
  | samp
  | strong
  | tt
  ;

